#
"""
Utils for activities associated with testing
[Interfaced as: SciPy functions]
"""
#
import numpy
from sklearn.metrics import r2_score


#


#
class TestSkeleton:

    def __init__(self, statistic, cdf):

        self.statistic = statistic
        self.cdf = cdf

    def _calculate_statistic(self, **kwargs):
        stated = self.statistic(**kwargs)
        return stated

    def _calculate_p_value(self, **kwargs):
        stated = self._calculate_statistic(**kwargs)
        p_valued = self.cdf(x=stated)
        return p_valued


def concassive_shiels(label_array, values_array, benchmark_label=0,
                      bins=None, binning_array=None, n_bins=10, woe=False):

    if bins is None:
        if binning_array is None:
            _, bins = numpy.histogram(values_array, bins=n_bins)
        else:
            _, bins = numpy.histogram(binning_array, bins=n_bins)

    # print()

    bins[0], bins[-1] = -numpy.inf, numpy.inf
    values_masks = [(bins[j] <= values_array) * (values_array < bins[(j + 1)]) for j in range((len(bins) - 1))]
    label_mask = label_array == benchmark_label

    # print()

    rates0 = numpy.array([label_array[(values_masks[j] * label_mask)].shape[0] / label_mask.sum()
                          for j in range(len(values_masks))])
    rates1 = numpy.array([label_array[(values_masks[j] * ~label_mask)].shape[0] / (~label_mask).sum()
                          for j in range(len(values_masks))])

    # print()

    if isinstance(woe, bool):
        if woe:
            result = numpy.log((rates0 / rates1))
        else:
            result = (rates0 - rates1) * numpy.log((rates0 / rates1))
    else:
        raise Exception("Misunderstood key value for 'which' = {0}".format(woe))

    inf_mask = (result == numpy.inf) + (result == -numpy.inf)
    result = result[~inf_mask]
    result = result[~numpy.isnan(result)]
    result = result.sum()

    return result


def bayes_correction(array, ct, dr):
    forward = array * (ct / dr)
    backward = (1 - array) * ((1 - ct) / (1 - dr))
    corrected = forward / (forward + backward)
    return corrected


def variance_inflation_measure(y_true, y_hat):
    r2_scored = r2_score(y_true=y_true, y_pred=y_hat)

    return 1 / (1 - r2_scored)
